package com.mowerm.opentools.clippy;

import java.io.*;
import java.util.*;

import com.borland.primetime.*;
import com.borland.primetime.editor.*;
import com.borland.primetime.properties.*;
import com.mowerm.opentools.clippy.actions.*;

/**
 *  <code>Clippy</code> is a JBuilder OpenTool that provides multiple clipboard
 *  buffers.<p>
 *
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 22, 2001
 *@version    1.0
 */

public class Clippy {
	/**
	 *  Description of the Field
	 */
	private static BufferManager bufferManager = null;

	/**
	 *  Description of the Field
	 */
	private static PrintWriter debug = null;


	/**
	 *  Materialize a new <code>MultipleClipboardBuffersOpenTool</code> .<p>
	 *
	 *
	 */
	private Clippy() {
	}


	/**
	 *  Sets the DebugModeEnabled attribute of the Clippy class
	 *
	 *@param  _enabled  The new DebugModeEnabled value
	 */
	public static void setDebugModeEnabled( boolean _enabled ) {
		ClippyPropertiesGroup.DEBUG_MODE_ENABLED.setBoolean( _enabled );
	}



	/**
	 *  Sets the CopyRingModeEnabled attribute of the Clippy class
	 *
	 *@param  _enabled  The new CopyRingModeEnabled value
	 */
	public static void setCopyRingModeEnabled( boolean _enabled ) {
		ClippyPropertiesGroup.COPY_RING_MODE_ENABLED.setBoolean( _enabled );
	}


	/**
	 *  Sets the PasteRingModeEnabled attribute of the Clippy class
	 *
	 *@param  _enabled  The new PasteRingModeEnabled value
	 */
	public static void setPasteRingModeEnabled( boolean _enabled ) {
		ClippyPropertiesGroup.PASTE_RING_MODE_ENABLED.setBoolean( _enabled );
	}


	/**
	 *  Sets the BufferManagerDialogWidth attribute of the Clippy class
	 *
	 *@param  _width  The new BufferManagerDialogWidth value
	 */
	public static void setBufferManagerDialogWidth( int _width ) {
		ClippyPropertiesGroup.BUFFER_MANAGER_DIALOG_WIDTH.setInteger( _width );
	}


	/**
	 *  Sets the BufferManagerDialogHeight attribute of the Clippy class
	 *
	 *@param  _height  The new BufferManagerDialogHeight value
	 */
	public static void setBufferManagerDialogHeight( int _height ) {
		ClippyPropertiesGroup.BUFFER_MANAGER_DIALOG_HEIGHT.setInteger( _height );
	}


	/**
	 *  Sets the BufferCount attribute of the Clippy class
	 *
	 *@param  _bufferCount  The new BufferCount value
	 */
	public static void setBufferCount( int _bufferCount ) {
		debug( "Setting buffer count = " + _bufferCount );
		getBufferManager().resizeBuffers( _bufferCount );
		ClippyPropertiesGroup.BUFFER_COUNT.setInteger( _bufferCount );
	}


	/**
	 *  Sets the BypassSystemClipboardModeEnabled attribute of the Clippy class
	 *
	 *@param  _enabled  The new BypassSystemClipboardModeEnabled value
	 */
	public static void setBypassSystemClipboardModeEnabled( boolean _enabled ) {
		ClippyPropertiesGroup.BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED.setBoolean( _enabled );
	}


	/**
	 *  Sets the BeepConfirmationModeEnabled attribute of the Clippy class
	 *
	 *@param  _enabled  The new BeepConfirmationModeEnabled value
	 */
	public static void setBeepConfirmationModeEnabled( boolean _enabled ) {
		ClippyPropertiesGroup.BEEP_CONFIRMATION_MODE_ENABLED.setBoolean( _enabled );
	}


	/**
	 *  Sets the AutoBindKeymapModeEnabled attribute of the Clippy class
	 *
	 *@param  _enabled  The new AutoBindKeymapModeEnabled value
	 */
	public static void setAutoBindKeymapModeEnabled( boolean _enabled ) {
		ClippyPropertiesGroup.AUTO_BIND_KEYMAP_MODE_ENABLED.setBoolean( _enabled );
	}


	/**
	 *  Gets the BufferManagerDialogWidth attribute of the Clippy class
	 *
	 *@return    The BufferManagerDialogWidth value
	 */
	public static int getBufferManagerDialogWidth() {
		return ClippyPropertiesGroup.BUFFER_MANAGER_DIALOG_WIDTH.getInteger();
	}


	/**
	 *  Gets the BufferManagerDialogHeight attribute of the Clippy class
	 *
	 *@return    The BufferManagerDialogHeight value
	 */
	public static int getBufferManagerDialogHeight() {
		return ClippyPropertiesGroup.BUFFER_MANAGER_DIALOG_HEIGHT.getInteger();
	}


	/**
	 *  Gets the AutoBindKeymapModeEnabled attribute of the Clippy class
	 *
	 *@return    The AutoBindKeymapModeEnabled value
	 */
	public static boolean isAutoBindKeymapModeEnabled() {
		return ClippyPropertiesGroup.AUTO_BIND_KEYMAP_MODE_ENABLED.getBoolean();
	}


	/**
	 *  Gets the DebugModeEnabled attribute of the Clippy class
	 *
	 *@return    The DebugModeEnabled value
	 */
	public static boolean isDebugModeEnabled() {
		return ClippyPropertiesGroup.DEBUG_MODE_ENABLED.getBoolean();
	}


	/**
	 *  Returns the instance that manages Clippy's buffers.<p>
	 *
	 *
	 *
	 *@return    Instance of <code>BufferManager</code> .
	 */
	public static synchronized BufferManager getBufferManager() {
		if( bufferManager == null ) {
			bufferManager = new BufferManager();
		}
		return bufferManager;
	}


	/**
	 *  Returns <code>true</code> if Clippy's copy-ring-mode is enabled.<p>
	 *
	 *  When enabled copy operations treat the buffers as a ring.<p>
	 *
	 *
	 *
	 *@return    <code>true</code> if Clippy is in copy-ring-mode.
	 */
	public static boolean isCopyRingModeEnabled() {
		return ClippyPropertiesGroup.COPY_RING_MODE_ENABLED.getBoolean();
	}


	/**
	 *  Gets the PasteRingModeEnabled attribute of the Clippy class
	 *
	 *@return    The PasteRingModeEnabled value
	 */
	public static boolean isPasteRingModeEnabled() {
		return ClippyPropertiesGroup.PASTE_RING_MODE_ENABLED.getBoolean();
	}


	/**
	 *  Gets the BypassSystemClipboardModeEnabled attribute of the Clippy class
	 *
	 *@return    The BypassSystemClipboardModeEnabled value
	 */
	public static boolean isBypassSystemClipboardModeEnabled() {
		return ClippyPropertiesGroup.BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED.getBoolean();
	}


	/**
	 *  Gets the BeepConfirmationModeEnabled attribute of the Clippy class
	 *
	 *@return    The BeepConfirmationModeEnabled value
	 */
	public static boolean isBeepConfirmationModeEnabled() {
		return ClippyPropertiesGroup.BEEP_CONFIRMATION_MODE_ENABLED.getBoolean();
	}


	/**
	 *  Gets the BufferCount attribute of the Clippy class
	 *
	 *@return    The BufferCount value
	 */
	public static int getBufferCount() {
		return ClippyPropertiesGroup.BUFFER_COUNT.getInteger();
	}


	/**
	 *  Initialize the OpenTool and its keymaps.<p>
	 *
	 *
	 *
	 *@param  _majorVersion  Primetime version number.
	 *@param  _minorVersion  Primetime API version number.
	 */
	public static void initOpenTool( byte _majorVersion, byte _minorVersion ) {
		printNotices();
		if( !passesVersionCheck( _majorVersion, _minorVersion ) ) {
			debug( "OpenTools version check failed.  Need API version 4 or above!" );
			return;
		}

		// Get the Clippy properties accessible
		registerPropertyGroup();

		// Make sure the buffers are ready
		getBufferManager();

		// Bind the keys used to invoke Clippy - unless user is managing bindings themselves
		bindKeys();

		// Bind a listener so that Clippy knows when JB is shutting down
		bindListener();
	}


	/**
	 *  Description of the Method
	 */
	public static void registerPropertyGroup() {
		PropertyManager.registerPropertyGroup( new ClippyPropertiesGroup() );
	}


	/**
	 *  Description of the Method
	 *
	 *@param  _message  Description of Parameter
	 */
	public static void debug( String _message ) {
		if( Clippy.isDebugModeEnabled() ) {
			if( debug == null ) {
				initDebug();
			}
			debug.println( _message );
			debug.flush();
			System.err.println( _message );
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  _message  Description of Parameter
	 *@param  _error    Description of Parameter
	 */
	public static void debug( String _message, Throwable _error ) {
		debug( _message );
		_error.printStackTrace( debug );
		debug.flush();
		_error.printStackTrace();
	}


	/**
	 *  Description of the Method
	 */
	public static void bindKeys() {
		HashMap _bindings = new HashMap();
		_bindings.put( "copy-to-buffer", new CopyToBufferAction() );
		_bindings.put( "cut-to-buffer", new CutToBufferAction() );
		_bindings.put( "paste-from-buffer", new PasteFromBufferAction() );
		_bindings.put( "select-buffer", new SelectBufferAction() );
		_bindings.put( "select-buffer-0", new SelectBufferXAction( 0 ) );
		_bindings.put( "select-buffer-1", new SelectBufferXAction( 1 ) );
		_bindings.put( "select-buffer-2", new SelectBufferXAction( 2 ) );
		_bindings.put( "select-buffer-3", new SelectBufferXAction( 3 ) );
		_bindings.put( "select-buffer-4", new SelectBufferXAction( 4 ) );
		_bindings.put( "select-buffer-5", new SelectBufferXAction( 5 ) );
		_bindings.put( "select-buffer-6", new SelectBufferXAction( 6 ) );
		_bindings.put( "select-buffer-7", new SelectBufferXAction( 7 ) );
		_bindings.put( "toggle-ring-mode", new ToggleRingModeAction() );
		_bindings.put( "import-system-clipboard-to-buffer", new ImportSystemClipboardToBufferAction() );
		KeyBindingsManager _bindingsManager = new KeyBindingsManager( _bindings );

		// Register the editor actions so that users can rebind them
		_bindingsManager.registerBindings();

		// If Clippy is managing key bindings...
		if( isAutoBindKeymapModeEnabled() ) {
			// Register with the editor manager so that when it sets the key map
			// we can install our bindings.  This also covers situations where the
			// user changes to another keymap.
			EditorManager.addPropertyChangeListener( _bindingsManager );
		}
	}


	/**
	 *  Description of the Method
	 */
	public static void bindListener() {
		ClippyBrowserListener.register();
	}


	/**
	 *  Description of the Method
	 */
	protected static void initDebug() {
		try {
			debug = new PrintWriter( new BufferedWriter( new FileWriter( new File( System.getProperty( "user.home" ), "clippy.debug" ) ) ) );
			debug.println( "Clippy version 1.06" );
			debug.println( "--[ System Properties ]----" );
			System.getProperties().list( debug );
			debug.println();
			debug.println( "--[ Messages ]----" );
			debug.flush();
		} catch( Exception ex ) {
			System.err.println( "Clippy : Error : Cannot initialize debug capture file" );
		}
	}


	/**
	 *  Determine whether the current version of JBuilder is the same or higher
	 *  than the version that OpenTool was written for. We hope that JBuilder will
	 *  retain enough downward compatibility not to break this opentool...<p>
	 *
	 *
	 *
	 *@param  _majorVersion  Primetime version number.
	 *@param  _minorVersion  Primetime API version number.
	 *@return                <code>true</code> if the PrimeTime version number
	 *      passes the check.
	 */
	private static boolean passesVersionCheck( byte _majorVersion, byte _minorVersion ) {
		return _majorVersion >= PrimeTime.CURRENT_MAJOR_VERSION;
	}


	/**
	 *  Print a startup notice for the JBuilder log.<p>
	 *
	 *
	 */
	private static void printNotices() {
		System.err.println( "**********************************************************************" );
		System.err.println( "** Initializing:  Clippy - The Multiple Buffer Clipboard OpenTool   **" );
		System.err.println( "**                Version 1.06                                      **" );
		System.err.println( "**                By Matt Mower <matt_mower@hotmail.com>            **" );
		System.err.println( "**********************************************************************" );
	}

}
