package com.mowerm.opentools.clippy;

import com.borland.primetime.editor.*;
import com.borland.primetime.properties.*;

/**
 *  <code>ClippyPropertiesGroup</code> represents the configurable properties of
 *  Clippy.<p>
 *
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 26, 2001
 *@version    1.0
 */

public class ClippyPropertiesGroup implements PropertyGroup {
	/**
	 *  Topic object associated with the properties
	 */
	public final static Object CLIPPY_TOPIC = new Object();

	/**
	 *  Topic description
	 */
	public final static String CLIPPY = "Clippy";

	/**
	 *  Name of copy-ring-mode property
	 */
	public final static String OPTION_COPY_RING_MODE_ENABLED = "copy.ring.mode";

	/**
	 *  Name of paste-ring-mode property
	 */
	public final static String OPTION_PASTE_RING_MODE_ENABLED = "paste.ring.mode";

	/**
	 *  Description of the Field
	 */
	public final static String OPTION_BEEP_CONFIRMATION_MODE_ENABLED = "beep.confirmation.mode";

	/**
	 *  Description of the Field
	 */
	public final static String OPTION_AUTO_BIND_KEYMAP = "auto.bind.keymap.mode";

	/**
	 *  Name of buffer-count property
	 */
	public final static String OPTION_BUFFER_COUNT = "buffer.count";

	/**
	 *  Name of bypass-system-clipboard mode property
	 */
	public final static String OPTION_BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED = "bypass.system.clipboard.mode";

	/**
	 *  Description of the Field
	 */
	public final static String OPTION_DEBUG_MODE_ENABLED = "debug.mode";

	/**
	 *  Description of the Field
	 */
	public final static String OPTION_BUFFER_MANAGER_DIALOG_WIDTH = "buffer.manager.dialog.width";

	/**
	 *  Description of the Field
	 */
	public final static String OPTION_BUFFER_MANAGER_DIALOG_HEIGHT = "buffer.manager.dialog.height";

	/**
	 *  Description of the Field
	 */
	public final static GlobalBooleanProperty DEBUG_MODE_ENABLED = new GlobalBooleanProperty( CLIPPY, OPTION_DEBUG_MODE_ENABLED, false );

	/**
	 *  Property for copy-ring-mode
	 */
	public final static GlobalBooleanProperty COPY_RING_MODE_ENABLED = new GlobalBooleanProperty( CLIPPY, OPTION_COPY_RING_MODE_ENABLED, false );

	/**
	 *  Property for paste-ring-mode
	 */
	public final static GlobalBooleanProperty PASTE_RING_MODE_ENABLED = new GlobalBooleanProperty( CLIPPY, OPTION_PASTE_RING_MODE_ENABLED, false );

	/**
	 *  Property for buffer-count
	 */
	public final static GlobalIntegerProperty BUFFER_COUNT = new GlobalIntegerProperty( CLIPPY, OPTION_BUFFER_COUNT, 8 );

	/**
	 *  Description of the Field
	 */
	public final static GlobalIntegerProperty BUFFER_MANAGER_DIALOG_WIDTH = new GlobalIntegerProperty( CLIPPY, OPTION_BUFFER_MANAGER_DIALOG_WIDTH, 800 );

	/**
	 *  Description of the Field
	 */
	public final static GlobalIntegerProperty BUFFER_MANAGER_DIALOG_HEIGHT = new GlobalIntegerProperty( CLIPPY, OPTION_BUFFER_MANAGER_DIALOG_HEIGHT, 600 );

	/**
	 *  Description of the Field
	 */
	public final static GlobalBooleanProperty BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED = new GlobalBooleanProperty( CLIPPY, OPTION_BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED, false );

	/**
	 *  Description of the Field
	 */
	public final static GlobalBooleanProperty BEEP_CONFIRMATION_MODE_ENABLED = new GlobalBooleanProperty( CLIPPY, OPTION_BEEP_CONFIRMATION_MODE_ENABLED, false );

	/**
	 *  Description of the Field
	 */
	public final static GlobalBooleanProperty AUTO_BIND_KEYMAP_MODE_ENABLED = new GlobalBooleanProperty( CLIPPY, OPTION_AUTO_BIND_KEYMAP, true );


	/**
	 *  Materialize a new <code>ClippyPropertiesGroup</code> .<p>
	 *
	 *
	 */
	public ClippyPropertiesGroup() {
	}


	/**
	 *  Gets the PageFactory attribute of the ClippyPropertiesGroup object
	 *
	 *@param  _topic  Description of Parameter
	 *@return         The PageFactory value
	 */
	public PropertyPageFactory getPageFactory( Object _topic ) {
		if( _topic == EditorManager.EDITOR_TOPIC ) {
			return
				new PropertyPageFactory( "Clippy", "Multiple Clipboard Buffers" ) {
					/**
					 *  Description of the Method
					 *
					 *@return    Description of the Returned Value
					 */
					public PropertyPage createPropertyPage() {
						return new ClippyPropertiesPage();
					}
				};
		}
		return null;
	}


	/**
	 *  Description of the Method
	 */
	public void initializeProperties() {
		Clippy.debug( "Clippy properties:" );
		Clippy.debug( OPTION_DEBUG_MODE_ENABLED + " = " + DEBUG_MODE_ENABLED.getBoolean() );
		Clippy.debug( OPTION_COPY_RING_MODE_ENABLED + " = " + COPY_RING_MODE_ENABLED.getBoolean() );
		Clippy.debug( OPTION_PASTE_RING_MODE_ENABLED + " = " + PASTE_RING_MODE_ENABLED.getBoolean() );
		Clippy.debug( OPTION_BUFFER_COUNT + " = " + BUFFER_COUNT.getInteger() );
		Clippy.debug( OPTION_BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED + " = " + BYPASS_SYSTEM_CLIPBOARD_MODE_ENABLED.getBoolean() );
		Clippy.debug( OPTION_BEEP_CONFIRMATION_MODE_ENABLED + " = " + BEEP_CONFIRMATION_MODE_ENABLED.getBoolean() );
		Clippy.debug( OPTION_BUFFER_MANAGER_DIALOG_WIDTH + " = " + BUFFER_MANAGER_DIALOG_WIDTH.getInteger() );
		Clippy.debug( OPTION_BUFFER_MANAGER_DIALOG_HEIGHT + " = " + BUFFER_MANAGER_DIALOG_HEIGHT.getInteger() );
		Clippy.debug( OPTION_AUTO_BIND_KEYMAP + " = " + AUTO_BIND_KEYMAP_MODE_ENABLED.getBoolean() );
	}
}
