package com.mowerm.opentools.clippy;

import java.awt.event.*;
import java.beans.*;
import java.util.*;

import javax.swing.*;
import javax.swing.text.*;

import com.borland.primetime.editor.*;

/**
 *  If Clippy manages it's own key bindings this is the class that will do it.
 *  <p>
 *
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 22, 2001
 *@version    1.0
 */
public class KeyBindingsManager implements PropertyChangeListener {
	/**
	 *  Mapping from action-key to EditorAction
	 */
	private Map bindings;


	/**
	 *  Constructor for the KeyBindingsManager object
	 *
	 *@param  _bindings  Description of Parameter
	 */
	public KeyBindingsManager( Map _bindings ) {
		super();
		bindings = _bindings;
	}


	/**
	 *  Gets the action with a specified action-key.<p>
	 *
	 *
	 *
	 *@param  _actionKey  Action key
	 *@return             Specific action associated with that key
	 */
	public EditorAction getAction( String _actionKey ) {
		return ( EditorAction ) bindings.get( _actionKey );
	}


	/**
	 *  This method is called to add each of the Clippy bindings to the set of
	 *  known JBuilder IDE bindings. This should allow the user to manage bindings
	 *  themselves in the regular way as an alternative to having Clippy manage
	 *  them.<p>
	 *
	 *
	 */
	public void registerBindings() {
		for( Iterator _bindings = bindings.values().iterator(); _bindings.hasNext();  ) {
			EditorActions.addBindableEditorAction( ( EditorAction ) _bindings.next() );
		}
	}


	/**
	 *  Bind all Clippy keys to the specified <code>Keymap</code>.<p>
	 *
	 *
	 *
	 *@param  _keyMap  <code>Keymap</code> to bind keys to
	 */
	public void bindKeys( Keymap _keyMap ) {
		if( _keyMap != null ) {
			// Override bindings for Copy, Cut & Paste
			bindCopy( _keyMap );
			bindCut( _keyMap );
			bindPaste( _keyMap );
			bindSelect( _keyMap );
			Clippy.debug( "Clippy : Ok : Keymap bindings installed" );
		} else {
			Clippy.debug( "Clippy : Error : Cannot bind keys - invalid Keymap reference specified" );
		}

	}


	/**
	 *  Bind all copy keys to the keymap
	 *
	 *@param  _keyMap  Keymap to place new bindings in
	 */
	public void bindCopy( Keymap _keyMap ) {
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_COPY, 0 ), getAction( "copy-to-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_INSERT, KeyEvent.CTRL_MASK ), getAction( "copy-to-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_C, KeyEvent.CTRL_MASK ), getAction( "copy-to-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_INSERT, KeyEvent.ALT_MASK ), getAction( "import-system-clipboard-to-buffer" ) );
	}



	/**
	 *  Bind all cut keys to the keymap
	 *
	 *@param  _keyMap  Keymap to place new bindings in
	 */
	public void bindCut( Keymap _keyMap ) {
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_CUT, 0 ), getAction( "cut-to-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_DELETE, KeyEvent.SHIFT_MASK ), getAction( "cut-to-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_X, KeyEvent.CTRL_MASK ), getAction( "cut-to-buffer" ) );
	}


	/**
	 *  Bind all paste keys to the keymap
	 *
	 *@param  _keyMap  Keymap to place new bindings in
	 */
	public void bindPaste( Keymap _keyMap ) {
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_PASTE, 0 ), getAction( "paste-from-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_INSERT, KeyEvent.SHIFT_MASK ), getAction( "paste-from-buffer" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_V, KeyEvent.CTRL_MASK ), getAction( "paste-from-buffer" ) );
	}


	/**
	 *  Bind all selection keys to the keymap.
	 *
	 *@param  _keyMap  Keymap to place new bindings in
	 */
	public void bindSelect( Keymap _keyMap ) {
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_R, KeyEvent.CTRL_MASK | KeyEvent.SHIFT_MASK ), getAction( "toggle-ring-mode" ) );
		_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_B, KeyEvent.ALT_MASK ), getAction( "select-buffer" ) );

		if( Clippy.getBufferCount() > 0 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_1, KeyEvent.ALT_MASK ), getAction( "select-buffer-0" ) );
		}
		if( Clippy.getBufferCount() > 1 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_2, KeyEvent.ALT_MASK ), getAction( "select-buffer-1" ) );
		}
		if( Clippy.getBufferCount() > 2 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_3, KeyEvent.ALT_MASK ), getAction( "select-buffer-2" ) );
		}
		if( Clippy.getBufferCount() > 3 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_4, KeyEvent.ALT_MASK ), getAction( "select-buffer-3" ) );
		}
		if( Clippy.getBufferCount() > 4 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_5, KeyEvent.ALT_MASK ), getAction( "select-buffer-4" ) );
		}
		if( Clippy.getBufferCount() > 5 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_6, KeyEvent.ALT_MASK ), getAction( "select-buffer-5" ) );
		}
		if( Clippy.getBufferCount() > 6 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_7, KeyEvent.ALT_MASK ), getAction( "select-buffer-6" ) );
		}
		if( Clippy.getBufferCount() > 7 ) {
			_keyMap.addActionForKeyStroke( KeyStroke.getKeyStroke( KeyEvent.VK_8, KeyEvent.ALT_MASK ), getAction( "select-buffer-7" ) );
		}
	}


	/**
	 *  Called in the event that a JBuilder property changes. In this case we are
	 *  interested only if the keymap has been changed, in which case we wish to
	 *  rebind our keys.<p>
	 *
	 *
	 *
	 *@param  _event  Property change event
	 */
	public void propertyChange( PropertyChangeEvent _event ) {
		String _propertyName = _event.getPropertyName();

		// We are only interested in keymap changes
		if( _propertyName.equals( EditorManager.keymapAttribute ) ) {
			bindKeys( EditorManager.getKeymap() );
		}
	}

}

/*
 *  This class is autogenerated by ClassWizard, written by Joachim Nilsson in April/May 2001.
 *  This software is provided "as is" with no warranty of any kind.
 */
