package com.mowerm.opentools.clippy.actions;

import javax.swing.*;

import com.mowerm.opentools.clippy.*;

/**
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 22, 2001
 *@version    1.0
 */

public class CopyToBufferAction extends EditorAction {
	/**
	 *  Materialize a new <code>CopyToBufferAction</code> .<p>
	 *
	 *
	 */
	public CopyToBufferAction() {
		super( "copy-to-buffer" );
		putValue( Action.LONG_DESCRIPTION, "Copies currently selected text to the current buffer" );
		putValue( "ActionGroup", "Clippy" );
	}


	/**
	 *  Called when the key-binding for copy is invoked, safely invokes <code>doBufferCopy()</code>
	 *  .<p>
	 *
	 *
	 */
	public void editActionPerformed() {
		try {
			if( getEditorPane() != null ) {
				// Then copy to the currenly selected buffer
				doBufferCopy();
			} else {
				setStatus( " Clippy : Error in copy : Can't find editor pane to copy from!" );
			}
		} catch( Throwable _t ) {
			Clippy.debug( "Error in copy", _t );
			setStatus( " Clippy : Error in copy : " + _t.getClass().getName() + " : " + _t.getMessage() );
		}
	}


	/**
	 *  Copy the contents of the editor selection into a buffer.<p>
	 *
	 *
	 */
	public void doBufferCopy() {
		setStatus( " Copying to buffer: " + Clippy.getBufferManager().getBufferDisplayNumber() );

		// If we are in copy-ring-mode then advance the buffers so the copy operation goes into the next buffer,
		// preserving the content of the current buffer
		if( Clippy.isCopyRingModeEnabled() ) {
			Clippy.getBufferManager().advanceBuffers();
		}

		// Copy to the system-clipboard first
		getEditorPane().copy();

		// Copy the system clipboard into a buffer
		Clippy.getBufferManager().copySystemClipboardToBuffer();

		if( Clippy.isBeepConfirmationModeEnabled() ) {
			getEditorPane().getToolkit().beep();
		}
	}
}
