package com.mowerm.opentools.clippy.actions;

import javax.swing.*;

import com.mowerm.opentools.clippy.*;

/**
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 25, 2001
 *@version    1.0
 */

public class CutToBufferAction extends EditorAction {

	/**
	 *  Materialize a new <code>CutToBufferAction</code> .<p>
	 *
	 *
	 */
	public CutToBufferAction() {
		super( "cut-to-buffer" );
		putValue( Action.LONG_DESCRIPTION, "Cuts currently selected text to the current buffer" );
		putValue( "ActionGroup", "Clippy" );
	}


	/**
	 *  Called when the key-binding for cut is invoked, safely invokes <code>doBufferCut()</code>
	 *  .<p>
	 *
	 *
	 */
	public void editActionPerformed() {
		try {
			if( getEditorPane() != null ) {
				// Then copy to the currenly selected buffer
				doBufferCut();
			} else {
				setStatus( " Clippy : Error : Can't find editor pane to cut from!" );
			}
		} catch( Throwable _t ) {
			Clippy.debug( "Error in cut", _t );
			setStatus( " Clippy : Error in cut : " + _t.getClass().getName() + " : " + _t.getMessage() );
		}
	}


	/**
	 *  Cut the content of the editor selection into a buffer.<p>
	 *
	 *
	 */
	public void doBufferCut() {
		setStatus( " Cutting to buffer: " + Clippy.getBufferManager().getBufferDisplayNumber() );

		// If we are in copy-ring-mode then advance the buffer to cut into the next buffer, preserving
		// the current buffer
		if( Clippy.isCopyRingModeEnabled() ) {
			Clippy.getBufferManager().advanceBuffers();
		}

		// Copy to the system-clipboard first
		getEditorPane().cut();

		// Copy the system clipboard into a buffer
		Clippy.getBufferManager().copySystemClipboardToBuffer();

		if( Clippy.isBeepConfirmationModeEnabled() ) {
			getEditorPane().getToolkit().beep();
		}
	}

}
