package com.mowerm.opentools.clippy.actions;

import java.awt.event.*;

import javax.swing.*;

import com.borland.primetime.editor.*;
import com.borland.primetime.ide.*;
import com.borland.primetime.ui.*;

/**
 *  <code>EditorAction</code> is a custom version of JBuilder's own {@link
 *  com.borland.primetime.editor.EditorAction EditorAction} class. I forget now
 *  why it was required -- I think it is because the Borland class did not allow
 *  subclasses access to the invoking event.<p>
 *
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 25, 2001
 *@version    1.0
 */

public abstract class EditorAction extends com.borland.primetime.editor.EditorAction {
	/**
	 *  Underlying event
	 */
	private ActionEvent event;

	/**
	 *  A panel used for displaying messages
	 */
	private static JPanel messagePanel = new JPanel();


	/**
	 *  Materialize a new <code>EditorAction</code> .<p>
	 *
	 *
	 *
	 *@param  _name  Description of Parameter
	 */
	public EditorAction( String _name ) {
		super( _name );
	}


	/**
	 *  Set the contents of the status window for the browser.<p>
	 *
	 *
	 *
	 *@param  _statusMessage  Status message
	 */
	public void setStatus( String _statusMessage ) {
		getBrowser().getStatusView().setText( _statusMessage );
	}


	/**
	 *  Return the underlying <code>ActionEvent</code>.<p>
	 *
	 *
	 *
	 *@return    event
	 */
	public ActionEvent getEvent() {
		return event;
	}


	/**
	 *  Gets the EditorPane attribute of the EditorAction object
	 *
	 *@return    The EditorPane value
	 */
	public EditorPane getEditorPane() {
		return getEditorTarget( getEvent() );
	}


	/**
	 *  Gets the Browser attribute of the EditorAction object
	 *
	 *@return    The Browser value
	 */
	public Browser getBrowser() {
		return Browser.findBrowser( getEvent().getSource() );
	}


	/**
	 *  Squirrel away the <code>ActionEvent</code> object for later use before
	 *  invoking the <code>editActionPerformed()</code> method to allow subclass to
	 *  handle the event.<p>
	 *
	 *
	 *
	 *@param  _event  event
	 */
	public void actionPerformed( ActionEvent _event ) {
		event = _event;
		editActionPerformed();
	}


	/**
	 *  Display a message over the browser window.<p>
	 *
	 *
	 *
	 *@param  _message  Message to show
	 */
	public void message( String _message ) {
		DefaultDialog.showModalDialog( getBrowser(), _message, messagePanel, null );
	}



	/**
	 *  Subclasses should override this to handle the edit action event.<p>
	 *
	 *
	 */
	public abstract void editActionPerformed();
}
