package com.mowerm.opentools.clippy.actions;

import javax.swing.*;

import com.mowerm.opentools.clippy.*;

/**
 *  This source code is hereby placed in the public domain.<p>
 *
 *
 *
 *@author     Matt Mower <matt_mower@hotmail.com>
 *@created    June 25, 2001
 *@version    1.0
 */

public class PasteFromBufferAction extends EditorAction {
	/**
	 *  Materialize a new <code>PasteFromBufferAction</code> .<p>
	 *
	 *
	 */
	public PasteFromBufferAction() {
		super( "paste-from-buffer" );
		putValue( Action.LONG_DESCRIPTION, "Pastes text from the current buffer" );
		putValue( "ActionGroup", "Clippy" );
	}


	/**
	 *  Description of the Method
	 */
	public void editActionPerformed() {
		try {
			if( getEditorPane() != null && getEditorPane().isEnabled() && getEditorPane().isEditable() ) {
				doBufferPaste();
			} else {
				if( getEditorPane() == null ) {
					setStatus( " Clippy : Error: Can't find editor pane to paste to!" );
				} else {
					setStatus( " Clippy : Error : Current editor pane is disabled or read-only, cannot paste!" );
				}
			}
		} catch( Throwable _t ) {
			Clippy.debug( "Error in paste", _t );
			setStatus( " Clippy : Error in paste : " + _t.getClass().getName() + " : " + _t.getMessage() );
		}
	}


	/**
	 *  Description of the Method
	 */
	public void doBufferPaste() {
		setStatus( " Clippy : Pasting from buffer: " + Clippy.getBufferManager().getBufferDisplayNumber() );

		if( Clippy.isCopyRingModeEnabled() ) {
			directPaste();
		} else {
			// Put the buffer contents into the system clipboard
			Clippy.getBufferManager().copyBufferToSystemClipboard();

			// Do a standard paste
			getEditorPane().paste();
		}

		if( Clippy.isPasteRingModeEnabled() ) {
			Clippy.getBufferManager().selectNextBuffer();
		}

		if( Clippy.isBeepConfirmationModeEnabled() ) {
			getEditorPane().getToolkit().beep();
		}
	}


	/**
	 *  Description of the Method
	 */
	public void directPaste() {
		setStatus( " Clippy : Pasting directly from buffer: " + Clippy.getBufferManager().getBufferDisplayNumber() );

		String _contents = Clippy.getBufferManager().getBufferAsRawString();
		if( _contents != null ) {
			char _rawChars[] = _contents.toCharArray();
			char _convertedChars[] = new char[_rawChars.length];
			int _lastChar = 0;
			int _convertedIndex = 0;
			for( int _index = 0; _index < _rawChars.length; ++_index ) {
				char c = _rawChars[_index];
				if( c == '\n' && _lastChar == 13 ) {
					_lastChar = c;
					continue;
				}
				_lastChar = c;
				if( c == '\r' ) {
					c = '\n';
				}
				if( c == '\f' ) {
					c = '\n';
				}
				_convertedChars[_convertedIndex++] = c;
			}

			getEditorPane().replaceSelection( new String( _convertedChars, 0, _convertedIndex ) );
		}
	}
}
